@@ -1,4 +1,4 @@
-context("catalog_index")
+context("engine_index")
 
 data <- data.table::data.table(
   Max.X   = c(885228.88, 886993.96, 885260.93, 887025.96,
@@ -50,18 +50,27 @@ data <- data.table::data.table(
   filename = paste0("abc", 1:62)
 )
 
-ctg             <- new("LAScatalog")
-ctg@data        <- data
-ctg@proj4string <- sp::CRS("+init=epsg:26917")
+geom <- lapply(1:nrow(data), function(i)
+{
+  mtx <- matrix(c(data$Min.X[i], data$Max.X[i], data$Min.Y[i], data$Max.Y[i])[c(1, 1, 2, 2, 1, 3, 4, 4, 3, 3)], ncol = 2)
+  sf::st_polygon(list(mtx))
+})
 
-test_that("catalog_index finds files and makes correct chunks", {
+geom <-sf::st_sfc(geom)
+sf::st_crs(geom) <- 26917
+data <- sf::st_set_geometry(data, geom)
+
+ctg       <- new("LAScatalog")
+ctg@data  <- data
+
+test_that("engine_index finds files and makes correct chunks", {
 
   bboxes <- list(
-    raster::extent(890000, 890800, 630000, 630800),
-    raster::extent(890000 - 400, 890800 - 400, 630000 + 400, 630800 + 400)
+    sf::st_bbox(c(xmin = 890000, xmax = 890800, ymin = 630000, ymax = 630800)),
+    sf::st_bbox(c(xmin = 890000 - 400, xmax = 890800 - 400, ymin = 630000 + 400, ymax = 630800 + 400))
   )
 
-  clusters <- lidR:::catalog_index(ctg, bboxes)
+  clusters <- lidR:::engine_index(ctg, bboxes)
 
   expect_is(clusters, "list")
   expect_equal(length(clusters), 2L)
@@ -70,7 +79,7 @@ test_that("catalog_index finds files and makes correct chunks", {
   expect_equal(clusters[[2]]@files, c("abc18"))
   expect_equal(clusters[[2]]@filter, "-inside 889600 630400 890400 631200 ")
 
-  clusters <- lidR:::catalog_index(ctg, bboxes, lidR:::LIDRCIRCLE)
+  clusters <- lidR:::engine_index(ctg, bboxes, lidR:::LIDRCIRCLE)
 
   expect_is(clusters, "list")
   expect_equal(length(clusters), 2L)
@@ -80,14 +89,14 @@ test_that("catalog_index finds files and makes correct chunks", {
   expect_equal(clusters[[2]]@filter, "-inside_circle 890000 630800 400 ")
 })
 
-test_that("catalog_index returns NULL if there is no match", {
+test_that("engine_index returns NULL if there is no match", {
 
   bboxes <- list(
-    raster::extent(890000, 890800, 630000, 630800),
-    raster::extent(890000 - 400, 890800 - 400, 630000 - 2000, 630800 - 2000)
+    sf::st_bbox(c(xmin = 890000, xmax = 890800, ymin = 630000, ymax = 630800)),
+    sf::st_bbox(c(xmin = 890000 - 400, xmax = 890800 - 400, ymin = 630000 - 2000, ymax = 630800 - 2000))
   )
 
-  clusters <- lidR:::catalog_index(ctg, bboxes)
+  clusters <- lidR:::engine_index(ctg, bboxes)
 
   expect_is(clusters, "list")
   expect_equal(clusters[[1]]@files, c("abc12", "abc15", "abc18",  "abc21"))
